#ifndef FRVFILTER
#define FRVFILTER

#ifndef _POSIX_SOURCE
#define _POSIX_SOURCE
#endif
/*---------------------------------------------------------------------------*/
/*  FrvFilter.h   Dec 2, 2002 (from Siesta file USignal.*)                   */
/*---------------------------------------------------------------------------*/

#include "FrameL.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct FrvFilter2 FrvFilter2;

struct FrvFilter2 {
                             /*--------------------- input data -------------*/
  double a0;                 /* filter coefficients (see definition below)   */
  double a1;                 /* filter coefficients (see definition below)   */
  double a2;                 /* filter coefficients (see definition below)   */
  double b0;                 /* filter coefficients (see definition below)   */
  double b1;                 /* filter coefficients (see definition below)   */
  double b2;                 /* filter coefficients (see definition below)   */
  double reg0;               /* internal registers                           */
  double reg1;               /* internal registers                           */
  double reg2;               /* internal registers                           */
  double alfa0;              /* internal coefficients                        */
  double alfa1;              /* internal coefficients                        */
  double alfa2;              /* internal coefficients                        */
  double beta0;              /* internal coefficients                        */
  double beta1;              /* internal coefficients                        */
  double beta2;              /* internal coefficients                        */
  struct FrVect *output;     /* vector to hold the last data chunk           */
  char error[256];           /* hold error message if any                    */
  int nCall;                 /* number of call for the algorithms            */
  FrvFilter2* next;          /* to provide linnk list support                */
};

/*
  How does it work ? The transfer function (Laplace transform s=i*w) is:

                Yout(s)     a2 * s**2 + a1 * s + a0
                ------- =  -------------------------
                Yinp(s)     b2 * s**2 + b1 * s + b0

    Example 1 : first order low-pass filter at f0 with unity gain at dc
                a2=0.  a1=0.  a0=1.  b2=0.  b1=1/(2*pi*f0)  b0=1. 

    Example 2 : first order high-pass filter at f0 with unity gain at dc
                a2=0.  a1=1/(2*pi*f0)  a0=1.  b2=0.  b1=0.  b0=1.

    Example 3 : second order low-pass filter at f0 with quality factor Q 
                and unity gain at dc (e.g. pendulum)
                a2=0. a1=0. a0=1. b2=1/(2*pi*f0)**2  b1=1/(2*pi*f0*Q)  b0=1

    Example 4 : integrator with time constant t
                a2=0. a1=0. a0=1. b2=0. b1=t b0=0.
*/    
void        FrvFilter2Free(FrvFilter2* filter);
int         FrvFilter2Proc(FrvFilter2 *filter, FrVect *vect);
int         FrvFilter2Init(FrvFilter2* filter, FrVect *vect);
FrvFilter2* FrvFilter2New(double a2,double a1,double a0,
                          double b2,double b1,double b0);

#ifdef __cplusplus
}
#endif

#endif
